import React, { useRef, useState, useContext, forwardRef, useMemo, useLayoutEffect } from 'react'
import { ConfigProvider, message, Modal, Button } from 'antd'
import { ExclamationCircleOutlined } from '@ant-design/icons'
import Generator, { defaultSettings } from 'fr-generator'
import { settings, baseSettings, globalSettings } from './config'
import widgets from '../widgets'
import { saveTableConfig, GetTableConfigJson } from '../../apis/process'
import FormRender from '../FormRender'
import { isObject } from '../../utils'
import Drag from '../components/Drag'

const FormDesigner = (props, ref) => {

  const { tableName } = props

  const { getPrefixCls } = useContext(ConfigProvider.ConfigContext)
  const prefixCls = getPrefixCls('pandaXform')
  const [visible, setVisible] = useState(false)
  const [schema, setSchema] = useState({})
  const designerRef = useRef(null)
  const formRenderRef = useRef(null)

  const settingsParent = useMemo(() => {
    let settingsParent = []
    settings.forEach(v => {
      v.widgets.forEach(s => {
        s.schema.tableNameParent = tableName
      })
      settingsParent.push(v)
    })
    return settingsParent
  }, [settings, tableName])

  const getJSON = (json, tableName) => {
    let properties = json?.properties
    let parent = {}
    let paths = []
    let ungroupedChild = {}
    if (isObject(properties)) {
      for (let v in properties) {
        if (properties[v].type === 'object') {
          parent[v] = properties[v]
          if (v !== '未分组') {
            paths.push(v)
          }
          let child = properties[v]?.properties
          if (isObject(child)) {
            for (let s in child) {
              child[s].tableNameParent = tableName
              if (v === '未分组') {
                ungroupedChild[s] = child[s]
              }
            }
          }
        } else {
          ungroupedChild[v] = {
            ...properties[v],
            tableNameParent: tableName
          }
        }
      }
    }
    if (JSON.stringify(ungroupedChild) !== '{}') {
      parent['未分组'] = {
        title: "(未分组)",
        type: "object",
        collapsed: false,
        groupBase: {},
        properties: {
          ...ungroupedChild
        }
      }
      paths.push('未分组')
    }
    return {
      ...json,
      paths,
      properties: parent,
    }
  }

  const getTableConfig = async (tableName) => {
    const { code, data, msg } = await GetTableConfigJson({ tableName })
    if (code === 0) {
      if (data && typeof data === 'string') {
        let json = JSON.parse(data)
        designerRef?.current?.setValue(getJSON(json, tableName))
        setSchema(json)
      }
    } else {
      message.error(msg)
    }
  }

  useLayoutEffect(() => {
    if (tableName) {
      getTableConfig(tableName)
    }
  }, [tableName])

  const extraButtons = [
    false,
    false,
    true,
    true,
    {
      text: '清空',
      onClick: (e) => {
        Modal.confirm({
          title: '确定要清空该表单配置吗?',
          icon: <ExclamationCircleOutlined />,
          centered: true,
          cancelText: '取消',
          okText: '确定',
          onCancel: () => {
            console.log('取消清空')
          },
          onOk: () => {
            const schema = designerRef.current.getValue()
            let newschema = {}
            for (let key in schema) {
              if (key !== 'properties') {
                newschema[key] = schema[key]
              }
            }
            designerRef.current.setValue(newschema)
          }
        })
      },
    },
    {
      text: '预览',
      onClick: (e) => {
        setVisible(true)
        let json = getJSON(designerRef.current.getValue(), tableName)
        setSchema(json)
        designerRef.current.setValue(json)
      }
    },
    {
      text: '提交',
      onClick: async (value) => {
        const errors = designerRef.current.getErrorFields()
        let json = getJSON(designerRef.current.getValue(), tableName)
        designerRef.current.setValue(json)
        if (errors.length) {
          return message.error('请按照提示完善表单内容')
        }
        const { code, data, msg } = await saveTableConfig(json)
        if (code === 0) {
          message.info('保存成功')
        } else {
          message.error(msg)
        }
      }
    }
  ]

  const getFields = (schema) => {
    let fields = []
    let parent = schema?.properties
    if (isObject(parent)) {
      for (let v in parent) {
        let child = parent[v]?.properties
        if (isObject(child)) {
          for (let s in child) {
            fields.push(s)
          }
        }
      }
    }
    return fields
  }

  const onSchemaChange = (schema) => {
    let fields = getFields(schema)
    window.sessionStorage.setItem('designerRef', fields)
  }

  const testSubmit = async () => {
    const { formValue, relationForm, errors } = await formRenderRef?.current?.getValues()
    if (errors.length) {
      return message.error('表单校验未通过!')
    }
  }

  return (
    <div className={prefixCls} style={{ width: '100%', height: '100%', background: 'white' }}>
      <div style={{ height: '100%' }}>
        <Generator
          configProvider={{ prefixCls: prefixCls }}
          mapping={{
            object: 'Header',
          }}
          validation={true}
          ref={designerRef}
          extraButtons={extraButtons}
          widgets={widgets}
          settings={settingsParent}
          commonSettings={baseSettings}
          globalSettings={globalSettings}
          onSchemaChange={onSchemaChange}
        />
        <Drag
          title={tableName}
          width='80%'
          bodyStyle={{ height: '700px', overflow: 'auto' }}
          onOk={() => setVisible(false)}
          onCancel={() => setVisible(false)}
          visible={visible}
          getContainer={false}
          destroyOnClose
          footer={
            <div>
              <Button onClick={() => setVisible(false)}>取消</Button>
              <Button onClick={testSubmit}>测试提交</Button>
            </div>
          }
        >
          <FormRender ref={formRenderRef} schemaValues={{formJson: schema }} />
        </Drag>
      </div>
    </div>
  )

}

export default forwardRef(FormDesigner)